// main.c -- Main procedure

#include <stdlib.h>
#include <string.h> // strtok
#include <math.h>
#include "video.h"
#include "proto.h"
#include "util.h"
#include "dither.h"

#include "ewinease.h"


static VidStream *theStream;

int appexit=0, firstrun=1;
void *win, *image;
ave_avo_win_std_dialog_getgadgets_t gdg;

char *titlename="Amiga MPEG-Player V1.02";


extern void ave_avo_gdg_std_filedlg(char *, char *, char *, int) __attribute__
((
	"qcall ave/avo/gdg/std/filedlg"
));


int my_ewe_event_handler(void *win)
{
   ave_avo_app_getevent_t eventIN;

   eventIN.targetavo = NULL;
   eventIN.eventmsg = NULL;
   eventIN.eventtype = 0;

   while (1)
   {
     eventIN = ave_avo_app_getevent(__EWE_INITPTRS.app, 0);

     if (eventIN.targetavo)
     {
       ave_avo_event(eventIN.targetavo, eventIN.eventmsg, eventIN.eventtype);
	
       if(_EWE_EVENT_DEBUG) ewe_debug_event_signal(eventIN);

       if(eventIN.eventtype != 0 &&  eventIN.targetavo != NULL)
       {
         ave_freeevent(eventIN.targetavo, eventIN.eventmsg);
     
         if (eventIN.eventtype==EV_QUIT)
           appexit=1;
       }
     }
     else break;
  }  
}



// Define buffer length.
#define BUF_LENGTH 80000

// External declaration of main decoding call.
extern VidStream *mpegVidRsrc();
extern VidStream *NewVidStream();

// Declaration of global variable to hold dither info.

#ifdef DCPREC
// Declaration of global variable to hold DC precision
int dcprec = 0;
#endif

// Global file pointer to incoming data.
FILE *input;
char *inputName=NULL;

// Quiet flag.
int quietFlag = 0;

// Display image on screen?
int noDisplayFlag = 0;
int loopFlag = 0;

/* Seek Value.
0 means do not seek.
N (N>0) means seek to N after the header is parsed
N (N<0) means the seek has beeen done to offset N
*/

/* Framerate, -1: specified in stream (default)
0: as fast as possible
N (N>0): N frames/sec 
*/
int framerate = 0;

// Flags/values to control Arbitrary start/stop frames.
int partialFlag = 0, startFrame = -1, endFrame = -1;

// Flag for gamma correction
int gammaCorrectFlag = 0;
double gammaCorrect = 1.0;

// Flag for chroma correction
int chromaCorrectFlag = 0;
double chromaCorrect = 1.0;

// Flag for high quality at the expense of speed
int qualityFlag = 0;

// no further error messages
static BOOLEAN exiting=FALSE;

// Brown - put X specific variables in xinfo struct
XInfo *xinfo;

// --- Prototypen
void CloseXDisplay(void);
void ResetVidStream(VidStream *vid);



// main -- Parses command line, starts decoding and displaying.
// Results:      None.
// Side effects: None.
//void 
main( int argc, char **argv)
{
  int mark;
  long seekValue=0; // holds value before it is put in vid_stream
  int owncmFlag=0;  // holds value before it is put in xinfo


  xinfo = (XInfo *) malloc(sizeof(XInfo));

  input = NULL;
  theStream = NULL;
  LUM_RANGE = 8;
  CR_RANGE = CB_RANGE = 4;
  noDisplayFlag = 0;

  printf("\n**** %s ****\n\n", titlename);
  printf("Ported over by Stefan Robl <stefan@qdev.de>\n");
  printf("Web (Updates, more stuff, ...): www.qdev.de\n\n");
  printf("This program is based upon the following project/source:\n\n");
  printf("       MPEG Video Software Decoder - Version 2.3; March 1996\n");
  printf("Lawrence A. Rowe, Ketan Patel, Brian Smith, Steve Smoot, and Eugene Hung\n");
  printf("     Computer Science Division-EECS, Univ. of Calif. at Berkeley\n\n");
  fflush(stdout);

  if (argc>=2)
    inputName=argv[1];

  if (!inputName)
  {
    printf("Please specify a MPEG video file!\n"); fflush(stdout);
  }

  if (!inputName)
  {
    char *fn=malloc(2);
    char *fp=malloc(2);
    char *ff=malloc(2);
    int s=0;
   
    strcpy(fn,"");
    strcpy(fp,"/");
    strcpy(ff,"*");
    printf("Opening File-Requester...\n"); fflush(stdout);
    ave_avo_gdg_std_filedlg((void *) fn,(void *) fp,(void *) ff, s);

    if (*fn!=0)
    {
      inputName=malloc(strlen(fp)+strlen(fn)+1);
      strcpy(inputName, fp);
      strcat(inputName, fn);
    }
    else exit(0); 
  }


  if ( !(input=fopen(inputName, "r")) )
  {
    fprintf(stderr, "Could not open file \"%s\"!\n", inputName); fflush(stderr);
    exit(0);
  }
  
  lum_values = (int *) malloc(LUM_RANGE*sizeof(int));
  cr_values  = (int *) malloc(CR_RANGE *sizeof(int));
  cb_values  = (int *) malloc(CB_RANGE *sizeof(int));
  

  ewe_init();

  init_tables();
  xinfo->owncmFlag = owncmFlag;
  xinfo->name = inputName;
  xinfo->ditherType = PPM_DITHER;
  
  wpixel[0] = 0xff0000;
  wpixel[1] = 0xff00;
  wpixel[2] = 0xff;
  xinfo->depth = 24;
  InitColorDither(1);
  
  //printf("Stage 1..\n");

  InitCrop();
    
  theStream = NewVidStream((unsigned int) BUF_LENGTH);
  theStream->input     = input;
  theStream->seekValue = seekValue;
  theStream->filename  = inputName;
  theStream->ditherType    = xinfo->ditherType;
  theStream->matched_depth = xinfo->depth;
  
  if (mpegVidRsrc(0, theStream, 1, xinfo)==NULL)
  {  
    // stream has already been destroyed
    theStream=NULL;
    fprintf(stderr, "Skipping movie \"%s\" - not an MPEG stream\n", inputName);
    fclose(input);
  }
 
  if (!theStream)
    exit(0);

  // Start time for each movie - do after windows are mapped
  theStream->realTimeStart = ReadSysClock();
  

  do
  {
      if (appexit)
        theStream->film_has_ended=TRUE; 

      mark = theStream->totNumFrames;
      // make sure we do a whole frame
      while (mark == theStream->totNumFrames)
        mpegVidRsrc(0, theStream, 0, xinfo);
      
      if (theStream->film_has_ended)
      {
        if (loopFlag)
        {
          clear_data_stream(theStream);
          // Reinitialize vid_stream pointers
          ResetVidStream(theStream);
          rewind(theStream->input);
          if (theStream->seekValue < 0)
            theStream->seekValue = 0 - theStream->seekValue;
              
          // Process start codes
          if (mpegVidRsrc(0, theStream, 1, xinfo)==NULL)
          {
            // print something sensible here, but we only get here if the file is
            // changed while we are decoding, right?
          }
        }
      }
  } 
  while (theStream!=NULL && !theStream->film_has_ended);

  if (theStream)
    PrintTimeInfo(theStream);

  ewe_shutdownAVE();
}




/*--------------------------------------------------------------
* DoDitherImage --
* Called when image needs to be dithered. Selects correct
* dither routine based on info in xinfo[0].ditherType.
* Results: None.
* Side effects: None.
*--------------------------------------------------------------
*/
void DoDitherImage(VidStream *vid_stream)
{
  Color32DitherImage(vid_stream->current->luminance, 
                     vid_stream->current->Cr, 
                     vid_stream->current->Cb, 
                     vid_stream->current->display,
                     (int) vid_stream->mb_height * 16,
                     (int) vid_stream->mb_width * 16);
}


unsigned int *imagebuffer=NULL;

void ExecutePPM(VidStream *vid_stream)
{
  register unsigned int *ptr, *p;
  register long n;

  if (firstrun)
  {
     win  = ewe_openWindow(titlename, vid_stream->h_size, vid_stream->v_size, FDI_INNER|FDI_CLOSE|FDI_BORDER|FDI_TITLE|FDI_CONTENT|FDI_DRAG);
     
     imagebuffer=(unsigned int *) calloc(vid_stream->h_size*vid_stream->v_size, sizeof(unsigned int));
     image = ave_avo_pix_32bit_open((void *) imagebuffer ,vid_stream->h_size,vid_stream->v_size,vid_stream->h_size*4);
   
     gdg = ave_avo_win_std_dialog_getgadgets(win);
     ave_avo_add(gdg.content, image, 0);
     ave_avo_pix_fbox_new(image, 0, 0, 0, vid_stream->h_size, vid_stream->v_size);
     ewe_showWindow(win);
     ewe_addEvent(EV_QUIT ,win,_EWE_NO_ROUTINE, _EWE_EVENT_RETURN);
     
     fflush(stdout);
     printf("Resolution (width x height):   %d x %d\n", vid_stream->h_size, vid_stream->v_size);
     fflush(stdout);
     firstrun=0;
  }
 
  n=vid_stream->h_size*vid_stream->v_size;
  p = (unsigned int *) vid_stream->current->display;
  ptr=imagebuffer;
  while (n-->0)
    *ptr++=*p++;

  my_ewe_event_handler(win);

  ewe_refreshWindow(win);
}
